/*
 *	aegis - project change supervisor
 *	Copyright (C) 1999 Peter Miller;
 *	All rights reserved.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 * MANIFEST: functions for reading input and filtering out CR LF sequences
 */

#include <ac/stdio.h> /* for EOF */

#include <error.h>
#include <input/crlf.h>
#include <input/private.h>


typedef struct input_crlf_ty input_crlf_ty;
struct input_crlf_ty
{
	input_ty	inherited;
	input_ty	*fp;
	int		pushback;
};


static void destruct _((input_ty *));

static void
destruct(p)
	input_ty	*p;
{
	input_crlf_ty	*this;

	this = (input_crlf_ty *)p;
	input_delete(this->fp);
}


static int get _((input_ty *));

static int
get(p)
	input_ty	*p;
{
	input_crlf_ty	*this;
	int		c;

	this = (input_crlf_ty *)p;
	if (this->pushback != EOF)
	{
		c = this->pushback;
		this->pushback = EOF;
		return c;
	}
	c = input_getc(this->fp);
	if (c == '\r')
	{
		c = input_getc(this->fp);
		if (c != '\n')
		{
			input_ungetc(this->fp, c);
			c = '\r';
		}
	}
	return c;
}


static void unget _((input_ty *, int));

static void
unget(p, c)
	input_ty	*p;
	int		c;
{
	input_crlf_ty	*this;

	if (c == EOF)
		return;
	this = (input_crlf_ty *)p;
	this->pushback = (unsigned char)c;
}


static const char *name _((input_ty *));

static const char *
name(p)
	input_ty	*p;
{
	input_crlf_ty	*this;

	this = (input_crlf_ty *)p;
	return input_name(this->fp);
}


static input_vtbl_ty vtbl =
{
	sizeof(input_crlf_ty),
	destruct,
	get,
	unget,
	name,
};


input_ty *
input_crlf(fp)
	input_ty	*fp;
{
	input_ty	*result;
	input_crlf_ty	*this;

	result = input_new(&vtbl);
	this = (input_crlf_ty *)result;
	this->fp = fp;
	this->pushback = EOF;
	return result;
}
