/*
 *	aegis - project change supervisor
 *	Copyright (C) 1994, 1995, 1998, 1999 Peter Miller;
 *	All rights reserved.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 * MANIFEST: functions for reading input from files
 */

#include <error.h>
#include <fopen_nfs.h>
#include <glue.h>
#include <input/file.h>
#include <input/private.h>
#include <mem.h>
#include <sub.h>

typedef struct input_file_ty input_file_ty;
struct input_file_ty
{
	input_ty	inherited;
	FILE		*fp;
	char		*fn;
};


static void destruct _((input_ty *));

static void
destruct(p)
	input_ty	*p;
{
	input_file_ty	*this;

	this = (input_file_ty *)p;
	if (glue_fclose(this->fp))
	{
		sub_context_ty	*scp;

		scp = sub_context_new();
		sub_errno_set(scp);
		sub_var_set(scp, "File_Name", "%s", this->fn);
		fatal_intl(scp, i18n("close $filename: $errno"));
		/* NOTREACHED */
	}
	mem_free(this->fn);
}


static int get _((input_ty *));

static int
get(p)
	input_ty	*p;
{
	input_file_ty	*this;
	int		c;

	this = (input_file_ty *)p;
	c = glue_fgetc(this->fp);
	if (c == EOF && glue_ferror(this->fp))
	{
		sub_context_ty	*scp;

		scp = sub_context_new();
		sub_errno_set(scp);
		sub_var_set(scp, "File_Name", "%s", this->fn);
		fatal_intl(scp, i18n("read $filename: $errno"));
		/* NOTREACHED */
	}
	return c;
}


static void unget _((input_ty *, int));

static void
unget(p, c)
	input_ty	*p;
	int		c;
{
	input_file_ty	*this;

	this = (input_file_ty *)p;
	if (c != EOF)
		glue_ungetc(c, this->fp);
}


static const char *name _((input_ty *));

static const char *
name(p)
	input_ty	*p;
{
	input_file_ty	*this;

	this = (input_file_ty *)p;
	return this->fn;
}


static input_vtbl_ty vtbl =
{
	sizeof(input_file_ty),
	destruct,
	get,
	unget,
	name,
};


input_ty *
input_file_open(fn)
	const char	*fn;
{
	input_ty	*result;
	input_file_ty	*this;

	result = input_new(&vtbl);
	this = (input_file_ty *)result;
	this->fp = fopen_with_stale_nfs_retry(fn, "rb");
	if (!this->fp)
	{
		sub_context_ty	*scp;

		scp = sub_context_new();
		sub_errno_set(scp);
		sub_var_set(scp, "File_Name", "%s", fn);
		fatal_intl(scp, i18n("open $filename: $errno"));
		/* NOTREACHED */
	}
	this->fn = mem_copy_string(fn);
	return result;
}
